/// <reference types="node" />

import type { IncomingMessage } from 'http';
import type { ServerResponse } from 'http';

/** @public */
export declare type AuthFlowType = 'auth-code' | 'device-auth';

/**
 * Create a new OIDC plugin instance that can be passed to the Node.js MongoDB
 * driver's MongoClientOptions struct.
 *
 * This plugin instance can be passed to multiple MongoClient instances.
 * It caches credentials based on cluster ID and username. If no username is
 * provided when connecting to the MongoDB instance, the cache will be shared
 * across all MongoClients that use this plugin instance.
 *
 * @public
 */
export declare function createMongoDBOIDCPlugin(options: Readonly<MongoDBOIDCPluginOptions>): MongoDBOIDCPlugin;

/**
 * Information that the application needs to show to users when using the
 * Device Authorization flow.
 *
 * @public
 */
export declare interface DeviceFlowInformation {
    verificationUrl: string;
    userCode: string;
}

/** @public */
export declare class MongoDBOIDCError extends Error {
    /* Excluded from this release type: [MongoDBOIDCErrorTag] */
    constructor(message: string);
    static isMongoDBOIDCError(value: unknown): value is MongoDBOIDCError;
}

/* Excluded from this release type: MongoDBOIDCErrorTag */

/** @public */
export declare interface MongoDBOIDCLogEventsMap {
    'mongodb-oidc-plugin:local-redirect-accessed': (event: {
        id: string;
    }) => void;
    'mongodb-oidc-plugin:oidc-callback-accepted': (event: {
        method: string;
        hasBody: boolean;
        errorCode?: string;
    }) => void;
    'mongodb-oidc-plugin:oidc-callback-rejected': (event: {
        method: string;
        hasBody: boolean;
        errorCode: string;
        isAcceptedOIDCResponse: boolean;
    }) => void;
    'mongodb-oidc-plugin:unknown-url-accessed': (event: {
        method: string;
        path: string;
    }) => void;
    'mongodb-oidc-plugin:local-listen-started': (event: {
        url: string;
    }) => void;
    'mongodb-oidc-plugin:local-listen-failed': (event: {
        url: string;
    }) => void;
    'mongodb-oidc-plugin:local-listen-succeeded': (event: {
        url: string;
        interfaces: string[];
    }) => void;
    'mongodb-oidc-plugin:local-server-close': (event: {
        url: string;
    }) => void;
    'mongodb-oidc-plugin:open-browser': (event: {
        customOpener: boolean;
    }) => void;
    'mongodb-oidc-plugin:notify-device-flow': () => void;
    'mongodb-oidc-plugin:auth-attempt-started': (event: {
        flow: string;
    }) => void;
    'mongodb-oidc-plugin:auth-attempt-succeeded': () => void;
    'mongodb-oidc-plugin:auth-attempt-failed': (event: {
        error: string;
    }) => void;
    'mongodb-oidc-plugin:refresh-failed': (event: {
        error: string;
    }) => void;
    'mongodb-oidc-plugin:skip-auth-attempt': (event: {
        reason: string;
    }) => void;
    'mongodb-oidc-plugin:auth-failed': (event: {
        error: string;
    }) => void;
    'mongodb-oidc-plugin:auth-succeeded': (event: {
        hasRefreshToken: boolean;
        expiresAt: string | null;
    }) => void;
}

/** @public */
export declare interface MongoDBOIDCPlugin {
    /**
     * A subset of MongoClientOptions that need to be set in order
     * for the MongoClient to an instance of this plugin.
     *
     * This object should be deep-merged with other, pre-existing
     * MongoClient driver options.
     */
    readonly mongoClientOptions: {
        readonly authMechanismProperties: {
            readonly REQUEST_TOKEN_CALLBACK: OIDCRequestFunction;
            readonly REFRESH_TOKEN_CALLBACK: OIDCRefreshFunction;
        };
    };
    /**
     * The logger instance passed in the options, or a default one otherwise.
     */
    readonly logger: TypedEventEmitter<MongoDBOIDCLogEventsMap>;
}

/** @public */
export declare interface MongoDBOIDCPluginOptions {
    /**
     * A local URL to listen on. If this is not provided, a default URL
     * standardized for MongoDB applications is used.
     *
     * This is only used when the Authorization Code flow is enabled,
     * and when it is possible to open a browser.
     */
    redirectURI?: string;
    /**
     * A function that opens an URL in a browser window. If this is `false`,
     * then all flows involving automatic browser operation (currently
     * Authorization Code flow) are disabled.
     *
     * If a `{ command: string }` object is provided, `command` will be spawned
     * inside a shell and receive the target URL as an argument. If `abortable`
     * is set, then a possible AbortSignal will be passed on and the child
     * process will be killed once that is reached. (This does not typically
     * make sense for GUI browsers, but can for command-line browsers.)
     *
     * If this option is missing or undefined, the default behavior is to use
     * `shell.openExternal()` if this is running inside of electron, and
     * the `open` package otherwise.
     */
    openBrowser?: undefined | false | {
        command: string;
        abortable?: boolean;
    } | ((options: OpenBrowserOptions) => Promise<OpenBrowserReturnType>);
    /**
     * The maximum time that the plugin waits for an opened browser to access
     * the URL that was passed to it, in milliseconds. The default is 10 seconds.
     * Passing a value of zero will disable the timeout altogether.
     */
    openBrowserTimeout?: number;
    /**
     * A callback to provide users with the information required to operate
     * the Device Authorization flow.
     */
    notifyDeviceFlow?: (information: DeviceFlowInformation) => Promise<void> | void;
    /**
     * Restrict possible OIDC authorization flows to a subset.
     */
    allowedFlows?: AuthFlowType[];
    /**
     * An optional EventEmitter that can be used for recording log events.
     */
    logger?: TypedEventEmitter<MongoDBOIDCLogEventsMap>;
    /**
     * An AbortSignal that can be used to explicitly cancel authentication
     * attempts, for example if a user intentionally aborts a connection
     * attempt.
     *
     * Note that the driver also registers its own AbortSignal with individual
     * authentication attempts in order to enforce a timeout, which has the
     * same effect for authentication attempts from that driver MongoClient
     * instance (but does not prevent other MongoClients from using this
     * plugin instance to authenticate).
     */
    signal?: OIDCAbortSignal;
    /**
     * A custom handler for providing HTTP responses for requests to the
     * redirect HTTP server used in the Authorization Code Flow.
     *
     * The default handler serves simple text/plain messages.
     */
    redirectServerRequestHandler?: RedirectServerRequestHandler;
}

/** @public */
export declare type OIDCAbortSignal = {
    aborted: boolean;
    reason?: unknown;
    addEventListener(type: 'abort', callback: () => void, options?: {
        once: boolean;
    }): void;
    removeEventListener(type: 'abort', callback: () => void): void;
};

/**
 * A copy of the Node.js driver's `OIDCMechanismServerStep1`
 * @public
 */
export declare interface OIDCMechanismServerStep1 {
    authorizationEndpoint?: string;
    tokenEndpoint?: string;
    deviceAuthorizationEndpoint?: string;
    clientId: string;
    clientSecret?: string;
    requestScopes?: string[];
}

/**
 * A copy of the Node.js driver's `OIDCRefreshFunction`
 * @public
 */
export declare type OIDCRefreshFunction = (principalName: string | undefined, idl: OIDCMechanismServerStep1, result: OIDCRequestTokenResult, abortSignal?: OIDCAbortSignal | number) => Promise<OIDCRequestTokenResult>;

/**
 * A copy of the Node.js driver's `OIDCRequestFunction`
 * @public
 */
export declare type OIDCRequestFunction = (principalName: string | undefined, idl: OIDCMechanismServerStep1, abortSignal?: OIDCAbortSignal | number) => Promise<OIDCRequestTokenResult>;

/**
 * A copy of the Node.js driver's `OIDCRequestTokenResult`
 * @public
 */
export declare interface OIDCRequestTokenResult {
    accessToken: string;
    expiresInSeconds?: number;
    refreshToken?: string;
}

/** @public */
export declare interface OpenBrowserOptions {
    /**
     * The URL to open the browser with.
     */
    url: string;
    /**
     * A signal that is aborted when the user or the driver abort
     * an authentication attempt.
     */
    signal: AbortSignal;
}

/** @public */
export declare type OpenBrowserReturnType = void | undefined | TypedEventEmitter<{
    exit(exitCode: number): void;
    error(err: unknown): void;
}>;

/** @public */
export declare type RedirectServerRequestHandler = (data: RedirectServerRequestInfo) => void;

/** @public */
export declare type RedirectServerRequestInfo = {
    /** The incoming HTTP request. */
    req: IncomingMessage;
    /** The outgoing HTTP response. */
    res: ServerResponse;
    /** The suggested HTTP status code. For unknown-url, this is 404. */
    status: number;
} & ({
    result: 'rejected';
    /** Error information reported by the IdP as defined in RFC6749 section 4.1.2.1 */
    error?: string;
    /** Error information reported by the IdP as defined in RFC6749 section 4.1.2.1 */
    errorDescription?: string;
    /** Error information reported by the IdP as defined in RFC6749 section 4.1.2.1 */
    errorURI?: string;
} | {
    result: 'accepted';
} | {
    result: 'unknown-url';
});

/** @public */
export declare interface TypedEventEmitter<EventMap extends object> {
    on<K extends keyof EventMap>(event: K, listener: EventMap[K]): this;
    once<K extends keyof EventMap>(event: K, listener: EventMap[K]): this;
    emit<K extends keyof EventMap>(event: K, ...args: EventMap[K] extends (...args: infer P) => any ? P : never): unknown;
}

export { }
